/*
* File:     procstat.c
* Author:   Michael Chinn
* UDel ID:  mechinn
*/

/*
 * Add a procstat builtin command.
 * This command should take a PID as the only
 * argument and print out the size,
 * rss and other info that is available of the process.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "procstat.h"

#if defined(__SUNPRO_C)
    #include <sys/procfs.h>
    #include <sys/processor.h>
    #include <sys/procset.h>
    #include <sys/pset.h>
    typedef struct lwpsinfo {
       int pr_flag;             /* lwp flags (DEPRECATED: see below) */
       id_t pr_lwpid;           /* lwp id */
       uintptr_t pr_addr;       /* internal address of lwp */
       uintptr_t pr_wchan;      /* wait addr for sleeping lwp */
       char pr_stype;           /* synchronization event type */
       char pr_state;           /* numeric lwp state */
       char pr_sname;           /* printable character for pr_state */
       char pr_nice;            /* nice for cpu usage */
       short pr_syscall;        /* system call number (if in syscall) */
       char pr_oldpri;          /* pre-SVR4, low value is high priority */
       char pr_cpu;             /* pre-SVR4, cpu usage for scheduling */
       int pr_pri;              /* priority, high value = high priority */
       ushort_t pr_pctcpu;      /* % of recent cpu time used by this lwp */
       timestruc_t pr_start;    /* lwp start time, from the epoch */
       timestruc_t pr_time;     /* cpu time for this lwp */
       char pr_clname[PRCLSZ];  /* scheduling class name */
       char pr_name[PRFNSZ];    /* name of system lwp */
       processorid_t pr_onpro;  /* processor which last ran this lwp */
       processorid_t pr_bindpro;/* processor to which lwp is bound */
       psetid_t pr_bindpset;    /* processor set to which lwp is bound */
       lgrp_id_t pr_lgrp;       /* home lgroup */
    } lwpsinfo_t;

    typedef struct psinfo {
       int pr_flag;             /* process flags (DEPRECATED: see below) */
       int pr_nlwp;             /* number of active lwps in the process */
       int pr_nzomb;            /* number of zombie lwps in the process */
       pid_t pr_pid;            /* process id */
       pid_t pr_ppid;           /* process id of parent */
       pid_t pr_pgid;           /* process id of process group leader */
       pid_t pr_sid;            /* session id */
       uid_t pr_uid;            /* real user id */
       uid_t pr_euid;           /* effective user id */
       gid_t pr_gid;            /* real group id */
       gid_t pr_egid;           /* effective group id */
       uintptr_t pr_addr;       /* address of process */
       size_t pr_size;          /* size of process image in Kbytes */
       size_t pr_rssize;        /* resident set size in Kbytes */
       dev_t pr_ttydev;         /* controlling tty device (or PRNODEV) */
       ushort_t pr_pctcpu;      /* % of recent cpu time used by all lwps */
       ushort_t pr_pctmem;      /* % of system memory used by process */
       timestruc_t pr_start;    /* process start time, from the epoch */
       timestruc_t pr_time;     /* cpu time for this process */
       timestruc_t pr_ctime;    /* cpu time for reaped children */
       char pr_fname[PRFNSZ];   /* name of exec’ed file */
       char pr_psargs[PRARGSZ]; /* initial characters of arg list */
       int pr_wstat;            /* if zombie, the wait() status */
       int pr_argc;             /* initial argument count */
       uintptr_t pr_argv;       /* address of initial argument vector */
       uintptr_t pr_envp;       /* address of initial environment vector */
       char pr_dmodel;          /* data model of the process */
       lwpsinfo_t pr_lwp;       /* information for representative lwp */
       taskid_t pr_taskid;      /* task id */
       projid_t pr_projid;      /* project id */
       poolid_t pr_poolid;      /* pool id */
       zoneid_t pr_zoneid;      /* zone id */
       ctid_t pr_contract;      /* process contract id */
    } psinfo_t;
#endif

int procstat(int argc,char **argv, char** envp) {
    if (argc==2){
        int pidLen = strlen(argv[1]);
    #if defined(__SUNPRO_C)
            printf("System: SUNPRO_C\n");
            char psinfo[pidLen + 13];
            strcpy(psinfo,"/proc/");
            strncat (psinfo, argv[1], pidLen);
            strncat (psinfo, "/psinfo", 7);
            printf("psinfo: %s\n",psinfo);
            if (access(psinfo, R_OK) == -1) { //check if we are allowed to read it
                perror( "access" );
            } else {
                FILE *file = fopen(psinfo, "rb"); /* read from the file in binary mode */
                if ( file != NULL ) {
                  struct psinfo data;
                  while (fread(&data, sizeof(psinfo_t), 1, file) == 1 ) {
                     printf("process flags: %d\n", data.pr_flag);
                     printf("number of active lwps in the process: %d\n", data.pr_nlwp);
                     printf("number of zombie lwps in the process: %d\n", data.pr_nzomb);
                     printf("process id: %d\n", data.pr_pid);
                     printf("process id of parent: %d\n", data.pr_ppid);
                     printf("process id of process group leader: %d\n", data.pr_pgid);
                     printf("session id: %d\n", data.pr_sid);
                     printf("real user id: %d\n", data.pr_uid);
                     printf("effective user id: %d\n", data.pr_euid);
                     printf("real group id: %d\n", data.pr_gid);
                     printf("effective group id: %d\n", data.pr_egid);
                     printf("size of process image in Kbytes: %lu\n", data.pr_size);
                     printf("resident set size in Kbytes: %lu\n", data.pr_rssize);
                     printf("% of recent cpu time used by all lwps: %hu\n", data.pr_pctcpu);
                     printf("% of system memory used by process: %hu\n", data.pr_pctmem);
                     printf("name of execed file: %c\n", data.pr_fname[PRFNSZ]);
                     printf("initial characters of arg list: %c\n", data.pr_psargs[PRARGSZ]);
                     printf("if zombie, the wait() status: %d\n", data.pr_wstat);
                     printf("initial argument count: %d\n", data.pr_argc);
                     printf("data model of the process: %c\n", data.pr_dmodel);
                  }
                  fclose(file);
                }
            }
        #elif defined(__linux__) || defined(__CYGWIN__)
            printf("System: linux/CYGWIN\n");
            char statm[pidLen + 12];
            strcpy(statm,"/proc/");
            strncat (statm, argv[1], pidLen);
            strncat (statm, "/statm", 6);
            printf("statm: %s\n",statm);
            if (access(statm, R_OK) == -1) { //check if we are allowed to read it
                perror( "access" );
            } else {
                int file = open(statm, O_RDONLY);
                if ( file == -1 ) { //execute it
                    perror( "open" );
                } else {
                    char buffer[1];
                    int count=-1;
                    int readSig = 1;
                    while(readSig != 0){
                        readSig = read(file, buffer, 1);
                        if (readSig == -1) {
                            perror("read");
                            exit(1);
                        } else {
                            ++count;
                        }
                    }
                    if ( lseek(file,0,SEEK_SET) == -1){
                        perror("lseek");
                        exit(1);
                    }
                    char statinfo[count];
                    if (read(file, statinfo, count) == -1) {
                        perror("read");
                        exit(1);
                    }
                    printf("size: %s\n",strtok(statinfo, " "));
                    printf("resident: %s\n",strtok(NULL, " "));
                    printf("share: %s\n",strtok(NULL, " "));
                    printf("text: %s\n",strtok(NULL, " "));
                    printf("lib: %s\n",strtok(NULL, " "));
                    printf("data: %s\n",strtok(NULL, " "));
                    printf("dt: %s\n",strtok(NULL, " "));
                }
            }
        #else
            printf("System: Unknown, contact developer\n");
        #endif
    } else {
        printf("Usage: procstat PID\n");
    }
}
