/*
* File:     watchmail.c
* Author:   Michael Chinn
* UDel ID:  mechinn
*/

/*
 * This command takes as the first argument a name of a file, which must already
 * exist (give an error if it doesn't), to watch for new mail in.
 * It takes an optional second argument of "off" to turn off of watching
 * of mail for that file. It should start up a new thread using
 * pthread_create(3thr) for each file to watch for mail in
 * (the name of the file is passed as a parameter).
 * This function, to watch for new mail, should run a while ( 1 ) loop,
 * which will do a stat(2) to see if the file got bigger since the last check.
 * A sleep for 1 second should be in the loop. The st_size member of a stat
 * struct can be used to check the size of the file. When the file increases
 * in size, the thread for that file should print out:
 *
 * BEEP You have new mail in FILE at TIME
 * 
 * The BEEP should be done with a \a in printf().
 * FILE should be the name of the file that the thread is watching.
 * And TIME should be the current time in ctime(3C) format.
 * To get the current time use gettimeofday(3C) and pass tv_sec of
 * the timeval struct to ctime().
 * When the "off" parameter is given the thread that is watching that
 * file needs to be canceled with pthread_cancel(3thr).
 * A linked list of active threads and files associated with each one
 * should be maintained to support this. When an error condition happens
 * in a thread pthread_exit(3thr) should be used. Your program should
 * support watching multiple files.
 * A separate thread will run to check each file.
 * 
 * Note that this command will print when a file it is watching gets bigger,
 * which could be annoying when using the shell for normal functioning.
 * However if you have multiple windows open this could be useful to
 * run in one for notification.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <pthread.h>

#include "watchmail.h"

void *tstart(void *arg) {
    char* path = (char*)arg;
    if (access(path, R_OK) == -1) { //check if we are allowed to read it
        perror( "access" );
        pthread_exit(0);
    } else {
        struct stat mail;
        if(stat(path, &mail) == -1) {
            perror( "stat" );
            pthread_exit(0);
        } else {
            off_t mailSize = mail.st_size;
            while(1) {
                if (access(path, R_OK) == -1) { //check if we are allowed to read it
                    perror( "access" );
                    pthread_exit(0);
                } else {
                    if (stat(path, &mail) == -1) {
                        perror( "stat" );
                        pthread_exit(0);
                    } else if (mail.st_size > mailSize) {
                        clock_t currentTime = time(NULL);
                        printf("\aYou have new mail in %s at %s\n",path,ctime(&currentTime));
                        mailSize = mail.st_size;
                    }
                    sleep(1);
                }
            }
        }
    }
    pthread_exit(0);
}

int watchmail(int argc,char **argv, char** envp) {
    if (argc==2){
/*
        struct timeval *currentTimeStruct;
        if(gettimeofday(currentTimeStruct,NULL)==-1){
            perror("gettimeofday");
            exit(1);
        }
*/

        addMail(argv[1]);
        pthread_attr_init(&mailHead->attr);
        pthread_create(&mailHead->tid,&mailHead->attr,tstart,(void *)mailHead->path);
        
    } else if (argc==3) {
        if(!strncmp(argv[2],"off",3)){
            pthread_cancel(findMail(argv[1]));
        }
    } else {
        printf("Usage: watchmail file <off>\n");
    }
}

// add command to front of history linked list
void addMail(char* path) {
    struct mailList *temp; //create a temp linked list value
    
    temp=(struct mailList *)malloc(sizeof(struct mailList));
    temp->path = malloc(strlen(path)*sizeof(char));
    strcpy(temp->path,path);

    if (mailHead == NULL){
        //List is Empty
        mailHead=temp;
        mailHead->next=NULL;
    } else {
        temp->next=mailHead;
        mailHead=temp;
    }
}

pthread_t findMail(char* path) {
    struct mailList *temp;
    temp = mailHead;
    while( temp != NULL ) {
        if(!strcmp(temp->path,path)){
            printf("Stopping watchmail on %s\n",temp->path);
            return temp->tid;
        } else {
            temp = temp->next;
        }
    }
}

// free history memory
void freeMail() {
   struct mailList *temp;
   while( mailHead != NULL ) {
      temp = mailHead->next;
      free (mailHead->path);
      free( mailHead );
      mailHead = temp;
   }
}

